(*****************************************************************************
 *          HOL-TestGen - specification based test case generation
 *                                                                            
 * harness.sml - 
 * Copyright (C) 2005 Achim D. Brucker <brucker@inf.ethz.ch>   
 *                    Burkhart Wolff   <bwolff@inf.ethz.ch>    
 *                                                                            
 * This file is part of HOL-TestGen.                                         
 *                                                                            
 * HOL-TestGen is free software; you can redistribute it and/or modify it    
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)  
 * any later version.                                                         
 *                                                                            
 * HOL-TestGen is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.                                                              
 *                                                                            
 * You should have received a copy of the GNU General Public License along    
 * with this program; if not, write to the Free Software Foundation, Inc.,    
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.                  
 ******************************************************************************)

signature TESTHARNESS = 
sig  
    type testresult
    val check : (unit -> 'a option) 
		-> (unit -> bool) list -> (unit -> bool)  
		-> ((bool option * exn option) * 
		    (bool option * 'a  option * exn option))
    val printResult : ('a -> string) -> int 
		      -> ((bool option * exn option) * 
			  (bool option * 'a option * exn option)) 
		      -> testresult
    val printList : ('a -> string) 
		    -> ((bool option * exn option) * 
			(bool option * 'a option * exn option)) list 
		    -> unit
    val rsf :bool
end

structure TestHarness  : TESTHARNESS = 
struct
datatype testresult = SUCCESS | WARNING | FAILURE | ERROR | FATAL

val rsf = false;

fun init () = print "NOT YET IMPLEMENTED"
	      
fun check retval prelist testpred  = 
    let
	fun fst (x,y)= x;
	val pre = (SOME(foldl (fn (x,y) => x andalso y) true 
			      (map (fn x => (x () = true)) prelist)),NONE)
	    handle e => (NONE,SOME(e));	
	val post =  (SOME((testpred()=true)),retval(),NONE) 
	    handle e => (NONE,NONE,SOME(e));    
    in 
	(pre,post)
    end
    
fun printResultValue toString res = 
    (case res of
	 NONE   => "\n"
       | SOME r => ", result: "^(toString r)^"\n")
    
fun printResult toString num ((SOME(true),NONE),(SOME(true),res,NONE)) =   
    let
	val _ = print ("Test "^(Int.toString(num))^
		       " -     SUCCESS"^
		       (printResultValue toString res))
    in 
	SUCCESS
    end
  | printResult toString num ((SOME(true),NONE),(SOME(false),res,NONE)) =  
    let
	val _ = print ("Test "^(Int.toString(num))
		       ^" - *** FAILURE: post-condition false"^
		       (printResultValue toString res))
    in 
	FAILURE
    end
  | printResult toString num ((SOME(true),NONE),(NONE,res,SOME(e_post))) = 
    let
	val _ = print ("Test "^(Int.toString(num))
		       ^" - *** FAILURE: exception during post-condition\n")
    in 
	FAILURE
    end
  | printResult toString num ((SOME(false),NONE),(SOME(true),res,NONE))=  
    let
	val _ = print ("Test "^(Int.toString(num))
		       ^" -     WARNING: pre-condition false "^
		       "(post-condition true)"^(printResultValue toString res))
    in 
	WARNING
    end
  | printResult toString num ((SOME(false),NONE),(SOME(false),res,NONE)) =
    let
	val _ = print ("Test "^(Int.toString(num))
		       ^" - *   WARNING: pre-condition false "^
		       "(post-condition false)"
		       ^(printResultValue toString res))
    in 
	WARNING
    end
  | printResult toString num ((SOME(false),NONE),(NONE,res,SOME(e_post))) =
    let
	val _ = print ("Test "^(Int.toString(num))^
		       " - **  WARNING: pre-condition false "^
		       "(exception during post_condition)\n")
    in 
	WARNING
    end
  | printResult toString num ((NONE,SOME(e_pre)),(SOME(true),res,NONE)) =  
    let
	val _ = print ("Test "^(Int.toString(num))^
		       " - **  ERROR:exception during pre-condition "^
		       "(post-condition true)"^(printResultValue toString res))
    in 
	ERROR
    end
  | printResult toString num ((NONE,SOME(e_pre)),(SOME(false),res,NONE)) = 
    let
	val _ = print ("Test "^(Int.toString(num))^
		       " - **  ERROR: exception during pre-condition "^
	   "(post-condition false)"^(printResultValue toString res))
    in 
	ERROR
    end
  | printResult toString num ((NONE,SOME(e_pre)),(NONE,res,SOME(e_post))) =
    let
	val _ = print ("Test "^(Int.toString(num))^
		       " - **  ERROR: exception during pre-condition "^
		       "and post-condition\n")
    in 
	ERROR
    end
  | printResult toString num  _ =  
    let
	val _ = print ("\nTest "^(Int.toString(num))^
		       " - ****** FATAL ERROR ******\n\n");
    in 
	FATAL
    end
    
fun foldl_map _ (x, []) = (x, [])
  | foldl_map f (x, y :: ys) =
    let
        val (x', y') = f (x, y);
        val (x'', ys') = foldl_map f (x', ys);
    in (x'', y' :: ys') end;
    
fun filter (pred: 'a->bool) : 'a list -> 'a list =
    let fun filt [] = []
          | filt (x :: xs) = if pred x then x :: filt xs else filt xs
    in filt end;

fun printList resultToString results  = 
    let	
	val _ = print("\n\n")
	val _ = print( "Test Results:\n") 
	val _ = print( "=============\n") 
	val (num,testres) = (foldl_map (fn (num,res) => 
					   (num+1,(printResult 
						       resultToString 
						       num res) )) 
				       (0, results));
	val _ = print ("\n\n");
	val _ = print ("Summary:\n");
	val _ = print ("--------\n");
	val _ = print ("Number successful tests cases: ")
	val suc = length (filter (fn (r) => (r=SUCCESS)) testres)
	val _ = print (Int.toString(suc)^" of "^Int.toString(num)^" ")
	val _ = print ("(ca. "^Int.toString((suc * 100) div num)^"%)\n")
	val _ = print ("Number of warnings:             ")
	val warn = length (filter (fn (r) => (r=WARNING)) testres)
	val _ = print (Int.toString(warn)^" of "^Int.toString(num)^" ")
	val _ = print ("(ca. "^Int.toString((warn * 100) div num)^"%)\n")
	val _ = print ("Number of errors:               ")
	val err = length (filter (fn (r) => (r=ERROR)) testres)
	val _ = print (Int.toString(err)^" of "^Int.toString(num)^" ")
	val _ = print ("(ca. "^Int.toString((err * 100) div num)^"%)\n")
	val _ = print ("Number of failures:             ")
	val fail = length (filter (fn (r) => (r=FAILURE)) testres)
	val _ = print (Int.toString(fail)^" of "^Int.toString(num)^" ")
	val _ = print ("(ca. "^Int.toString((fail * 100) div num)^"%)\n")
	val _ = print ("Number of fatal errors:         ")
	val fatal = length (filter (fn (r) => (r=FATAL)) testres)
	val _ = print (Int.toString(fatal)^" of "^Int.toString(num)^" ")
	val _ = print ("(ca. "^Int.toString((fatal * 100) div num)^"%)\n")
	val _ = print ("\nOverall result: ")
	val _ = if(suc=num) 
		then print ("success")
		else print ("failed ")
	val _ = print ("\n===============\n\n")
    in
	()
    end
end 	




 
